void setup()
{
  
  size(1440,900,OPENGL);                      // setting up the size of window
  
  if (frame != null) {
    frame.setResizable(true);          // resizable window
  }
  
  //setting up the camera
  cam= new PeasyCam( this,0,0,0,1000);       
  cam.setMinimumDistance(-5000);        // how near the camera can get 0,0,0
  cam.setMaximumDistance(8000);       // how far the camera can travel

  //control P5 stuff      
  controlP5 = new ControlP5(this); 
  g3 = (PGraphics3D)g;     
  setupGUI();        
        
  background(240);                     //set background to white
  //noLoop();                            //to run draw() only once

  // initialize the 3D array with number of rows and columns
  dataMatrix = new float[numRowsGlobal][numCols][numDewey];
  
for ( int d = 0; d<numDewey; d++){
  myTable = new Table();                 //allocating memory to new table 
  //load the awake data set
  String filename =  "awake_dewey_" + deweyNames[d] + ".csv";
  myTable = loadTable(filename, "header");
  
  //header for col names  as follows:
  //barcode  deweyClass  weekday_num  week_num  awake  earliest_cout_hour  latest_cout_hour

  // assign these variables to contain the number of rows and columns from myTable
  numRows = myTable.getRowCount();
  numCols = myTable.getColumnCount();


  // copy everything from table into a 2D array

  maxAwake = 0; //find max awake value [minutes] for scaling of colorbar

  for ( int i = 0; i< numRows; i++)          // a for loop where i is set to 0  and increments all the way to numRows by 1
  {
    println(filename);
    for ( int j = 0; j< numCols; j++)// a for loop where j is set to 0  and increments all the way to numColumns by 1
    {
      dataMatrix[i][j][d] = myTable.getInt(i, j);      // copying the table integer value at mytable (i,j) position into the dataMatrix
      //print( dataMatrix[i][j][d] + " ");            // print out the value of dataMatrix
    }                  
    if ( dataMatrix[i][4][d] > maxAwake)      // this is an if statement which checks for the condition in brackets                                                         // if true, it executes the statements in brackets 
    {
      maxAwake = (int)dataMatrix[i][4][d];
    }
    //println();      // switch to next line in the prompt, improves legibility
  }

      
   println ( "maximum value is:"+ maxAwake);
   println ( "rowCount: "+ numRows);
   println ( "columnCount: "+ numCols);

}
  //get sorted week - weekday matrix from getWeekdayWeek function
  println("getting sorted week - weekday - dewey matrix...");
  dataAwake = getWeekWeekday(dataMatrix, 4); // 4 is colIndex for awake
  println("interpolating missing data...");
  dataAwake = interpMissing(dataAwake);

  dataEarliestCout = getWeekWeekday(dataMatrix, 5); // 4 is colIndex for earliest cout hour
  dataLatestCout = getWeekWeekday(dataMatrix, 6); // 4 is colIndex for earliest cout hour
}
     
   // Mapping function to convert SQL output to week-weekday matrix that is easier to plot
// This look up table could be generated much more efficiently, but given the size of the dataset it works OK.
int[][][] getWeekWeekday(float[][][]data, int colIndex) {
  int[][][] dataWeekWeekday = new int[numWeeks][numWeekdays][numDewey];
  for (int d=1; d<=numDewey; d++) {
  for (int w=1; w<=numWeeks; w++) {
    for (int wd=1; wd<=numWeekdays; wd++) {   
      dataWeekWeekday[w-1][wd-1][d-1] = 0;//zero initializer
      for (int i=0; i<data.length; i++) {//go through values in data file and sort into week weekday matrix
        if (((int)data[i][3][d-1] == w) && ((int)data[i][2][d-1] == wd)) {
          if ((int)data[i][colIndex][d-1] > dataWeekWeekday[w-1][wd-1][d-1]) {
            dataWeekWeekday[w-1][wd-1][d-1] = (int)data[i][colIndex][d-1];
          }
        }
      }
      print( dataWeekWeekday[w-1][wd-1][d-1] + " ");
    }
    println();      // switch to next line in the prompt, improves legibility
  }
  }
  return dataWeekWeekday;
}

// Interpolation function for missing values in data matrix
int[][][] interpMissing(int[][][]data) {
    for (int d=1; d<=numDewey; d++) {
  for (int w=1; w<=numWeeks; w++) {
    for (int wd=1; wd<=numWeekdays; wd++) {   
      if (data[w-1][wd-1][d-1] == 0) {
        println("zero value detected, interpolate with data for the same day from rest of the year");
        int wdaySum = 0;
        for (int w1=1; w1<=numWeeks; w1++) {
          wdaySum += data[w1-1][wd-1][d-1];
        }
        data[w-1][wd-1][d-1] = wdaySum/numWeeks;
      }
      print( data[w-1][wd-1][d-1] + " ");
    }
    println();      // switch to next line in the prompt, improves legibility
  }
    }
  return data;
}  
